
/* Name: main.c
 * Project: AVR USB driver for CDC interface on Low-Speed USB
 *              for ATtiny2313
 * Author: Osamu Tamura
 * Creation Date: 2007-10-03
 * Tabsize: 4
 * Copyright: (c) 2007-2009 by Recursion Co., Ltd.
 * License: Proprietary, free under certain conditions. See Documentation.
 *
 */

#include "fcpu.h"

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "buffer.h"

#define HID_REPORT_BUFFER_SIZE 8
static uchar reportBuffer[HID_REPORT_BUFFER_SIZE];
static uchar idleRate;
static uchar hidBytesReceied;
PROGMEM const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,                     // USAGE_PAGE (Generic Desktop)
    0x09, 0x00,                     // USAGE (Undefined)
    0xa1, 0x01,                     // COLLECTION (Application)
    0x15, 0x00,                     //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,               //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                     //   REPORT_SIZE (8-bit)
    
    0x85, 0x01,                        //   REPORT ID (1)
    0x95, HID_REPORT_BUFFER_SIZE,   //   REPORT_COUNT (HID_REPORT_BUFFER_SIZE)
    0x09, 0x00,                     //   USAGE (Undefined)
    0xb2, 0x02, 0x01,               //   FEATURE (Data,Var,Abs,Buf)
    
    0xc0                            // END_COLLECTION
};

uchar usbFunctionSetup(uchar data[8]) {
    usbRequest_t    *rq = (void *)data;
    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){
            // Read from device
            
            // Copy Transmit buffer to report buffer
            uchar lenght = bufferSize(), a;
            if(lenght == 0)
                return 0;
            if(lenght>=HID_REPORT_BUFFER_SIZE)
                lenght = HID_REPORT_BUFFER_SIZE-1;
            for(a=1; a<=lenght; a++)
                reportBuffer[a] = bufferRead();
            reportBuffer[0] = 0;
            
            return lenght+1;  // Return length
            
            
        } else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            // Write to device
            
            // How many bytes has been already received
            hidBytesReceied = 0;
            
            return USB_NO_MSG;  // USB_NO_MSG=0xff special value for usbFunctionWrite()
        } else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        } else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    } else{
        /* no vendor specific requests implemented */
    }
    return 0;
}
uchar usbFunctionWrite( uchar *data, uchar length ) {
    
    //  Postpone receiving next data if buffer is full
    //if( bufferFree()<=HID_REPORT_BUFFER_SIZE)
    //    usbDisableAllRequests();
    
    return 1;
}

static void hardwareInit(void)
{

    /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
    USBDDR    = 0;    /* we do RESET by deactivating pullup */
    usbDeviceDisconnect();
#else
    USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif

    wdt_reset();
    _delay_ms(250);

#ifdef USB_CFG_PULLUP_IOPORT
    usbDeviceConnect();
#else
    USBDDR    = 0;      /*  remove USB reset condition */
#endif

}


uchar rx_buf[8];

int main(void)
{
    wdt_enable(WDTO_1S);
    odDebugInit();
    hardwareInit();
    usbInit();
    bufferReset();
    
    // 1     : N = 1
    // 2     : N = 8
    // 3     : N = 64
    // 4     : N = 256
    // 5     : N = 1024
    TIFR |= 1<<OCF0A;
    OCR0A = 250-1;
    TCCR0A = (1<<WGM01) | (0<<WGM00);
    TCCR0B = 3;
    
    // 2500 Hz : 100 - 3
    // 2000 Hz : 125 - 3
    // 1250 Hz :  50 - 4
    // 1000 Hz : 250 - 3
    //  625 Hz : 100 - 4
    //  500 Hz : 125 - 4
    //  250 Hz : 250 - 4
    //  125 Hz : 125 - 5
    
    // f = F_CPU/(N*(1+OCR))
    // f = 16000000/(64*250) = 1000;
    
    DDRB &= ~1;
    ADMUX = (0<<ADLAR) | 1; // ADC1, PB2
    ADCSRB = 3; // OCR0A edge
    ADCSRA = (1<<ADEN) | (1<<ADATE) | (1<<ADIF) | 7; // 128 prescaller
    DIDR0 = 1<<ADC1D;
    // 125 kHz at 16 MHz, 93 kHz at 12 MHz
    
    sei();
    for(;;){    /* main event loop */
        wdt_reset();
        usbPoll();

        if (ADCSRA&(1<<ADIF)) {
            ADCSRA |= (1<<ADIF);
            TIFR |= 1<<OCF0A;
            uint16_t value = ADC;
            bufferWrite(value>>8);
            bufferWrite(value);
        }
        
        
    }
    return 0;
}

