#ifndef FFT_H
#define FFT_H

#include <math.h>
#include <stdlib.h>

typedef struct
{
    double Re;
    double Im;
} complex;

static complex conv_from_polar(double r, double radians) {
    complex result;
    result.Re = r * cos(radians);
    result.Im = r * sin(radians);
    return result;
}

static complex add(complex left, complex right) {
    complex result;
    result.Re = left.Re + right.Re;
    result.Im = left.Im + right.Im;
    return result;
}

static complex multiply(complex left, complex right) {
    complex result;
    result.Re = left.Re*right.Re - left.Im*right.Im;
    result.Im = left.Re*right.Im + left.Im*right.Re;
    return result;
}

static complex* DFT_naive(complex* x, int N) {
    complex* X = (complex*) malloc(sizeof(complex) * N);
    int k, n;
    for(k = 0; k < N; k++) {
        X[k].Re = 0.0;
        X[k].Im = 0.0;
        for(n = 0; n < N; n++) {
            X[k] = add(X[k], multiply(x[n], conv_from_polar(1, -2*M_PI*n*k/N)));
        }
    }
    free(x);
    return X;
}

static void FFT_CooleyTukey(complex* input, complex* output, int N1, int N2) {
    int k1, k2;

    /* Allocate columnwise matrix */
    complex** columns = (complex**) malloc(sizeof(complex*) * N1);
    for(k1 = 0; k1 < N1; k1++) {
        columns[k1] = (complex*) malloc(sizeof(complex) * N2);
    }

    /* Allocate rowwise matrix */
    complex** rows = (complex**) malloc(sizeof(complex*) * N2);
    for(k2 = 0; k2 < N2; k2++) {
        rows[k2] = (complex*) malloc(sizeof(complex) * N1);
    }

    /* Reshape input into N1 columns */
    for (k1 = 0; k1 < N1; k1++) {
        for(k2 = 0; k2 < N2; k2++) {
            columns[k1][k2] = input[N1*k2 + k1];
        }
    }

    /* Compute N1 DFTs of length N2 using naive method */
    for (k1 = 0; k1 < N1; k1++) {
        columns[k1] = DFT_naive(columns[k1], N2);
    }

    /* Multiply by the twiddle factors  ( e^(-2*pi*j/N * k1*k2)) and transpose */
    for(k1 = 0; k1 < N1; k1++) {
        for (k2 = 0; k2 < N2; k2++) {
            rows[k2][k1] = multiply(conv_from_polar(1, -2.0*M_PI*k1*k2/(N1*N2)), columns[k1][k2]);
        }
    }

    /* Compute N2 DFTs of length N1 using naive method */
    for (k2 = 0; k2 < N2; k2++) {
        rows[k2] = DFT_naive(rows[k2], N1);
    }

    /* Flatten into single output */
    for(k1 = 0; k1 < N1; k1++) {
        for (k2 = 0; k2 < N2; k2++) {
            output[N2*k1 + k2] = rows[k2][k1];
        }
    }

    /* Free all alocated memory except output and input arrays */
    for(k1 = 0; k1 < N1; k1++) {
        free(columns[k1]);
    }
    for(k2 = 0; k2 < N2; k2++) {
        free(rows[k2]);
    }
    free(columns);
    free(rows);
    return;
}

#endif // FFT_H
