#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "myqgraphicsview.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    hid_thread(new Thread())
{
    ui->setupUi(this);
    qRegisterMetaType<QVector<int>>("QVector<int>");
    connect(hid_thread, SIGNAL(info(const QString)), this, SLOT(on_info(const QString)));
    connect(hid_thread, SIGNAL(data(const QVector<int>)), this, SLOT(scope(const QVector<int>)));
    hid_thread->start();
    connect(&graphics_timer, SIGNAL(timeout()), this, SLOT(graphics_update()));
    graphics_timer.start(GRAPHICS_UPDATE_MS);

    scope_x = 0;
    scope_val = 0;

    fft_x = 0;
    fft_panel_height = 0;
    for (int i = 0; i < FFT_STAGE_NUM; i++)
        fft_stage_size[i] = 0;

    ui->graphicsView->setScene(&scene);
#if FILE_LOGGING
    file_log.setFileName(FILE_PATH);
    file_log.open(QIODevice::WriteOnly | QIODevice::Text);
#endif
}

MainWindow::~MainWindow()
{
#if FILE_LOGGING
    file_log.close();
#endif
    graphics_timer.stop();
    hid_thread->stop();
    delete ui;
}

void MainWindow::resizeEvent(QResizeEvent* event)
{
   QMainWindow::resizeEvent(event);
   resize_canvas();
}

void MainWindow::resize_canvas()
{
    width = ui->graphicsView->width()-CANVAS_MARGIN;
    if (width < CANVAS_MIN_SIZE_X)
        width = CANVAS_MIN_SIZE_X;
    height = ui->graphicsView->height()-CANVAS_MARGIN;
    if (height < CANVAS_MIN_SIZE_Y)
        height = CANVAS_MIN_SIZE_Y;
    scene.setSceneRect(0, 0, width, height);
    canvas = QPixmap(width, height).toImage();
    // Scope
    for (int y = 0; y < height; y++)
        for (int x = 0; x < width; x++)
            canvas.setPixel(x, y, 0);
    // Ossciloscope lines
    for (int x = 0; x < width; x++) {
        canvas.setPixel(x, height-(PAINT_MARGIN+SCOPE_HEIGHT), 0xFFAAAAAA);
        canvas.setPixel(x, height-(1+PAINT_MARGIN), 0xFFAAAAAA);
    }
    // FFT lines
    int f = 0;
    int y = height-(2*PAINT_MARGIN+SCOPE_HEIGHT)-1;
    while (y > PAINT_MARGIN) {
        if (f%2) {
            //small 5 Hz
            for (int x = PAINT_MARGIN+FFT_OFFSET/2; x < PAINT_MARGIN+FFT_OFFSET-5; x++)
                canvas.setPixel(x, y, 0xFFFFFFFF);
        } else {
            // big 10 Hz
            for (int x = PAINT_MARGIN; x < PAINT_MARGIN+FFT_OFFSET-5; x++)
                canvas.setPixel(x, y, 0xFFFFFFFF);
        }
        f++;
        // Hz = time in seconds
        // 8 stage per 250ms = 2s
        // 2s = 2 indexes per 1 hz
        // 2 pixels per index
        // line per 5 Hz
        y -= (5*(FFT_STAGE_NUM*FFT_STAGE_SIZE*FFT_PIXELS_PER_INDEX))/1000;
    }
    fft_panel_height = (height-(3*PAINT_MARGIN+SCOPE_HEIGHT))/FFT_PIXELS_PER_INDEX;
    fft_x = 0;
    scope_x = 0;
    scope_val = 0;
}

void MainWindow::on_info(const QString text)
{
    ui->label->setText(text);
}

void MainWindow::graphics_update()
{
    scene.clear();
    scene.addPixmap(QPixmap::fromImage(canvas));
}

void MainWindow::scope(const QVector<int> data)
{
    bool flip = ui->checkBox->isChecked();
    for (int v : data) {
        data_buffer[data_buffer_size] = v;
        data_buffer_size = (data_buffer_size+1)%DATA_BUFFER_SIZE;
        if (scope_val == SCOPE_STACKED_POINTS) {
            scope_x = (scope_x+1)%width;
            for (int i = height+1-(SCOPE_HEIGHT+PAINT_MARGIN); i < height-(1+PAINT_MARGIN); i++)
                canvas.setPixel(scope_x, i, 0xFF000000);
            scope_val = 0;
        }
#if FILE_LOGGING
        file_log.write((QString::number(v)+"\n").toLatin1());
#endif
        v -= 90;
        if (v < 0) v = 0;
        if (v > 699) v = 699;
        if (flip) v = 699-v;
        canvas.setPixel(scope_x, height-(2+PAINT_MARGIN+(int)(v*(SCOPE_HEIGHT-2.0)/700.0)), 0xFFFFFFFF);
        scope_val++;
    }
    if (data_buffer_size >= FFT_STAGE_SIZE)
        fft_update();
}

void MainWindow::fft_update()
{
    int i, j, k, sum, size;
    double s, d, e;
    double avg;

    // uart buffer
    sum = 0;
    size = 0; // 2000
    for (i = 1; i < FFT_STAGE_NUM; i++)
        size += fft_stage_size[i];
    for (i = 0; i < size; i++) {
        fft_buffer[i] = fft_buffer[i+fft_stage_size[0]];
        sum += fft_buffer[i];
    }
    for (i = 0; i < FFT_STAGE_NUM-1; i++)
        fft_stage_size[i] = fft_stage_size[i+1];

    // copy FFT_STAGE_SIZE data
    //fft_stage_size[i] = data_buffer_size;
    fft_stage_size[FFT_STAGE_NUM-1] = FFT_STAGE_SIZE;
    for (i = 0; i < fft_stage_size[FFT_STAGE_NUM-1]; i++) {
        fft_buffer[i+size] = data_buffer[i];
        sum += fft_buffer[i+size];
    }
    size += fft_stage_size[FFT_STAGE_NUM-1];
    avg = (double)sum/(double)size;

    // Keep additional data in buffer
    for (i = 0; i < data_buffer_size-FFT_STAGE_SIZE; i++)
        data_buffer[i] = data_buffer[i+FFT_STAGE_SIZE];
    data_buffer_size = std::max(data_buffer_size-FFT_STAGE_SIZE, 0);

    // Hamming window
    for (i = 0; i < size; i++) {
        s = sin(M_PI*i/(size-1));
        data_a[i].Re = ((double)fft_buffer[i]-avg)*(s*s);
        data_a[i].Im = 0.0;
    }

    // fft
    FFT_CooleyTukey(data_a, data_b, size/40, 40);

    // Find maximum value
    for (i = 0; i < fft_panel_height; i++) {
        data_a[i].Re = sqrt(data_b[i].Im*data_b[i].Im + data_b[i].Re*data_b[i].Re);
    }

    // Paint slices
    i = 0;
    j = height-(2*PAINT_MARGIN+SCOPE_HEIGHT)-FFT_PIXELS_PER_INDEX;
    while (j >= PAINT_MARGIN) {
        d = data_a[i].Re/10000.0;
        if (d > 0.9)
            d = 0.9;
        e = 1.0-d;
        k = ((int)(9.0*e*d*d*d*255.0)<<16) | ((int)(15.0*e*e*d*d*255.0)<<8) | (int)(8.5*e*e*e*d*255.0);
        for (int t = 0; t < FFT_PIXELS_PER_INDEX; t++)
            canvas.setPixel(fft_x+FFT_OFFSET+PAINT_MARGIN, j+t, 0xFF000000 | k);
        i++;
        j-= FFT_PIXELS_PER_INDEX;
    }
    fft_x = (fft_x+1)%(width-(FFT_OFFSET+PAINT_MARGIN));
}

// Clear
void MainWindow::on_pushButton_clicked()
{
    for (int y = PAINT_MARGIN; y < height-(PAINT_MARGIN+SCOPE_HEIGHT); y++)
        for (int x = FFT_OFFSET+PAINT_MARGIN; x < width; x++)
            canvas.setPixel(x, y, 0xFF000000);
    fft_x = 0;
}
